{% load i18n %}

<div class="form-group clearfix">
    <label class="col-md-3 control-label" for="fcName">{% translate "FC Name" %}</label>
    <div class="col-md-9">
        <input id="fcName" name="fcName" type="text" class="form-control input-md" value="{{ mainCharacter }}">
    </div>
</div>
