{% load i18n %}

<div class="form-group clearfix">
    <div class="col-md-3">&nbsp;</div>
    <div class="col-md-9">
        <div class="checkbox">
            <label for="prePing">
                <input type="checkbox" name="prePing" id="prePing" value="1"> {% translate "Pre-Ping" %}
            </label>
        </div>
    </div>
</div>
