{% load i18n %}

<div class="form-group clearfix">
    <label class="col-md-3 control-label" for="formupLocation">{% translate "Formup Location" %}</label>
    <div class="col-md-9">
        <input id="formupLocation" name="formupLocation" type="text" class="form-control input-md" list="formupLocationList">

        {% if fleet_formup_locations|length > 0 %}
            <datalist id="formupLocationList">
                {% for formup_location in fleet_formup_locations %}
                    <option value="{{ formup_location }}">
                {% endfor %}
            </datalist>
        {% endif %}
    </div>
</div>
