{% load i18n %}

<div class="form-group fleetpings-create-optimer clearfix" style="display: none;">
    <div class="col-md-3">&nbsp;</div>
    <div class="col-md-9">
        <div class="checkbox">
            <label for="createOptimer">
                <input type="checkbox" name="createOptimer" id="createOptimer" value="1">
                {% translate "Create Optimer" %}<br>
                <span class="small">{% translate "If this checkbox is active, a fleet operations timer for this pre-ping will be created." %}</span>
            </label>
        </div>

        <div class="fleetpings-create-optimer-message"></div>
    </div>
</div>
