{% load i18n %}

<div class="form-group clearfix">
    <label class="col-md-3 control-label" for="fleetComms">{% translate "Fleet Comms" %}</label>
    <div class="col-md-9">
        <input id="fleetComms" name="fleetComms" type="text" class="form-control input-md" list="fleetCommsList">

        {% if fleet_comms|length > 0 %}
            <datalist id="fleetCommsList">
                {% for fleet_comm in fleet_comms %}
                    <option value="{{ fleet_comm }}">
                {% endfor %}
            </datalist>
        {% endif %}
    </div>
</div>
